@@ -212,7 +212,7 @@
 #include "eXtransport.h"
 #include "jpipe.h"
 
-#define EXOSIP_VERSION "5.2.1"
+#define EXOSIP_VERSION "5.3.0"
 
 #ifdef HAVE_WINSOCK2_H
 #define SOCKET_TYPE SOCKET
@@ -632,7 +632,9 @@ struct eXosip_t {
   int outgoing_wake_lock_state;
   int incoming_wake_lock_state;
 
-  char sip_instance[37]; /* can only be used if ONE excontext is used for ONE registration only */
+  char sip_instance[256]; /* can only be used if ONE excontext is used for ONE registration only */
+  char co_register_extra_params[512]; /* some extra contact parameters for Contact in REGISTER "audio;video;mobility="fixed";+sip.message="TRUE";other-param=66372;" parameter must not contains initial ";" */
+  char co_dialog_extra_params[512];   /* some extra contact parameters for Contact in INVITE's dialog "audio;video;" parameter must not contains initial ";" */
   char default_contact_displayname[256];
   int opt_sessiontimers_force;
   int opt_force_connectionreuse;
@@ -667,6 +669,8 @@ int _eXosip_update_top_via(osip_message_t *sip);
 int _eXosip_request_add_via(struct eXosip_t *excontext, osip_message_t *request);
 
 void _eXosip_mark_all_registrations_expired(struct eXosip_t *excontext);
+void _eXosip_mark_all_transaction_force_send(struct eXosip_t *excontext, int socket);
+void _eXosip_mark_all_transaction_transport_error(struct eXosip_t *excontext, int socket);
 void _eXosip_mark_registration_expired(struct eXosip_t *excontext, const char *call_id);
 int _eXosip_mark_all_transaction_ready(struct eXosip_t *excontext, fd_set *osip_fdset, fd_set *osip_wrset, fd_set *osip_exceptset, int *osip_fd_table);
 #ifdef HAVE_SYS_EPOLL_H
