@@ -752,7 +752,7 @@ static int _udp_read_udp_main_socket(struct eXosip_t *excontext) {
               jr->stun_nport = nport;
               memcpy(jr->stun_ipbuf, ipbuf, sizeof(jr->stun_ipbuf));
               jr->pong_supported = 1;
-            } else if (jr->stun_nport != nport && osip_strcasecmp(jr->stun_ipbuf, ipbuf) != 0) {
+            } else if (jr->stun_nport != nport || osip_strcasecmp(jr->stun_ipbuf, ipbuf) != 0) {
               OSIP_TRACE(osip_trace(__FILE__, __LINE__, OSIP_INFO1, NULL, "[eXosip] [UDP] [STUN answer] received from [%s][%d] [length=%i] [NEW XOR=%s %i]\n", src6host, recvport, i, ipbuf, nport));
               jr->stun_nport = nport;
               memcpy(jr->stun_ipbuf, ipbuf, sizeof(jr->stun_ipbuf));
@@ -917,7 +917,7 @@ static int udp_tl_epoll_read_message(struct eXosip_t *excontext, int nfds, struc
 
 #endif
 
-static int udp_tl_read_message(struct eXosip_t *excontext, fd_set *osip_fdset, fd_set *osip_wrset) {
+static int udp_tl_read_message(struct eXosip_t *excontext, fd_set *osip_fdset, fd_set *osip_wrset, fd_set *osip_exceptset) {
   struct eXtludp *reserved = (struct eXtludp *) excontext->eXtludp_reserved;
 
   if (reserved == NULL) {
@@ -1132,185 +1132,11 @@ static int udp_tl_send_message(struct eXosip_t *excontext, osip_transaction_t *t
   if (i < OSIP_SUCCESS)
     return i;
 
-#if 0
-  i = -1;
-
-  if (tr == NULL) {
-    _eXosip_srv_lookup(excontext, sip, &naptr_record);
-
-    if (naptr_record != NULL) {
-      eXosip_dnsutils_dns_process(naptr_record, 1);
-
-      if (naptr_record->naptr_state == OSIP_NAPTR_STATE_NAPTRDONE || naptr_record->naptr_state == OSIP_NAPTR_STATE_SRVINPROGRESS)
-        eXosip_dnsutils_dns_process(naptr_record, 1);
-    }
-
-    if (naptr_record != NULL && naptr_record->naptr_state == OSIP_NAPTR_STATE_SRVDONE) {
-      /* 4: check if we have the one we want... */
-      if (naptr_record->sipudp_record.name[0] != '\0' && naptr_record->sipudp_record.srventry[naptr_record->sipudp_record.index].srv[0] != '\0') {
-        /* always choose the first here.
-           if a network error occur, remove first entry and
-           replace with next entries.
-         */
-        osip_srv_entry_t *srv;
-        int n = 0;
-
-        for (srv = &naptr_record->sipudp_record.srventry[naptr_record->sipudp_record.index]; n < 10 && naptr_record->sipudp_record.srventry[naptr_record->sipudp_record.index].srv[0];
-             srv = &naptr_record->sipudp_record.srventry[naptr_record->sipudp_record.index]) {
-          if (srv->ipaddress[0])
-            i = _eXosip_get_addrinfo(excontext, &addrinfo, srv->ipaddress, srv->port, IPPROTO_UDP);
-
-          else
-            i = _eXosip_get_addrinfo(excontext, &addrinfo, srv->srv, srv->port, IPPROTO_UDP);
-
-          if (i == 0) {
-            host = srv->srv;
-            port = srv->port;
-            break;
-          }
-
-          i = eXosip_dnsutils_rotate_srv(&naptr_record->sipudp_record);
-
-          if (i <= 0) {
-            return -1;
-          }
-
-          if (i >= n) {
-            return -1;
-          }
-
-          i = -1;
-          /* copy next element */
-          n++;
-        }
-      }
-    }
-
-    if (naptr_record != NULL && naptr_record->keep_in_cache == 0)
-      osip_free(naptr_record);
-
-    naptr_record = NULL;
-
-  } else {
-    naptr_record = tr->naptr_record;
-  }
-
-  if (naptr_record != NULL) {
-    /* 1: make sure there is no pending DNS */
-    eXosip_dnsutils_dns_process(naptr_record, 0);
-
-    if (naptr_record->naptr_state == OSIP_NAPTR_STATE_NAPTRDONE || naptr_record->naptr_state == OSIP_NAPTR_STATE_SRVINPROGRESS)
-      eXosip_dnsutils_dns_process(naptr_record, 0);
-
-    if (naptr_record->naptr_state == OSIP_NAPTR_STATE_UNKNOWN) {
-      /* fallback to DNS A */
-      if (naptr_record->keep_in_cache == 0)
-        osip_free(naptr_record);
-
-      naptr_record = NULL;
-
-      if (tr != NULL)
-        tr->naptr_record = NULL;
-
-      /* must never happen? */
-
-    } else if (naptr_record->naptr_state == OSIP_NAPTR_STATE_INPROGRESS) {
-      /* 2: keep waiting (naptr answer not received) */
-      return OSIP_SUCCESS + 1;
-
-    } else if (naptr_record->naptr_state == OSIP_NAPTR_STATE_NAPTRDONE) {
-      /* 3: keep waiting (naptr answer received/no srv answer received) */
-      return OSIP_SUCCESS + 1;
-
-    } else if (naptr_record->naptr_state == OSIP_NAPTR_STATE_SRVINPROGRESS) {
-      /* 3: keep waiting (naptr answer received/no srv answer received) */
-      return OSIP_SUCCESS + 1;
-
-    } else if (naptr_record->naptr_state == OSIP_NAPTR_STATE_SRVDONE) {
-      /* 4: check if we have the one we want... */
-      if (naptr_record->sipudp_record.name[0] != '\0' && naptr_record->sipudp_record.srventry[naptr_record->sipudp_record.index].srv[0] != '\0') {
-        /* always choose the first here.
-           if a network error occur, remove first entry and
-           replace with next entries.
-         */
-        osip_srv_entry_t *srv;
-        int n = 0;
-
-        if (MSG_IS_REGISTER(sip) || MSG_IS_OPTIONS(sip)) {
-          /* activate the failover capability: for no answer OR 503 */
-          if (naptr_record->sipudp_record.srventry[tr->naptr_record->sipudp_record.index].srv_is_broken.tv_sec > 0) {
-            naptr_record->sipudp_record.srventry[naptr_record->sipudp_record.index].srv_is_broken.tv_sec = 0;
-            naptr_record->sipudp_record.srventry[naptr_record->sipudp_record.index].srv_is_broken.tv_usec = 0;
-
-            if (eXosip_dnsutils_rotate_srv(&naptr_record->sipudp_record) > 0) {
-              OSIP_TRACE(osip_trace(__FILE__, __LINE__, OSIP_INFO1, NULL, "[eXosip] [UDP] [tid=%i] doing UDP failover -> [%s][%d]\n", tid,
-                                    naptr_record->sipudp_record.srventry[naptr_record->sipudp_record.index].srv, naptr_record->sipudp_record.srventry[naptr_record->sipudp_record.index].port));
-            }
-          }
-        }
-
-        for (srv = &naptr_record->sipudp_record.srventry[naptr_record->sipudp_record.index]; n < 10 && naptr_record->sipudp_record.srventry[naptr_record->sipudp_record.index].srv[0];
-             srv = &naptr_record->sipudp_record.srventry[naptr_record->sipudp_record.index]) {
-          if (srv->ipaddress[0])
-            i = _eXosip_get_addrinfo(excontext, &addrinfo, srv->ipaddress, srv->port, IPPROTO_UDP);
-
-          else
-            i = _eXosip_get_addrinfo(excontext, &addrinfo, srv->srv, srv->port, IPPROTO_UDP);
-
-          if (i == 0) {
-            host = srv->srv;
-            port = srv->port;
-            break;
-          }
-
-          i = eXosip_dnsutils_rotate_srv(&naptr_record->sipudp_record);
-
-          if (i <= 0) {
-            return -1;
-          }
-
-          if (i >= n) {
-            return -1;
-          }
-
-          i = -1;
-          /* copy next element */
-          n++;
-          OSIP_TRACE(osip_trace(__FILE__, __LINE__, OSIP_INFO1, NULL, "[eXosip] [UDP] [tid=%i] doing UDP failover -> [%s][%d]\n", tid,
-                                naptr_record->sipudp_record.srventry[naptr_record->sipudp_record.index].srv, naptr_record->sipudp_record.srventry[naptr_record->sipudp_record.index].port));
-        }
-      }
-
-    } else if (naptr_record->naptr_state == OSIP_NAPTR_STATE_NOTSUPPORTED || naptr_record->naptr_state == OSIP_NAPTR_STATE_RETRYLATER) {
-      /* 5: fallback to DNS A */
-      if (naptr_record->keep_in_cache == 0)
-        osip_free(naptr_record);
-
-      naptr_record = NULL;
-
-      if (tr != NULL)
-        tr->naptr_record = NULL;
-    }
-  }
-
-  /* if SRV was used, destination may be already found */
-  if (i != 0) {
-    i = _eXosip_get_addrinfo(excontext, &addrinfo, host, port, IPPROTO_UDP);
-  }
-#endif
-
   i = _eXosip_get_addrinfo(excontext, &addrinfo, host, port, IPPROTO_UDP);
 
   if (i != 0) {
-    if (naptr_record != NULL) {
-      /* rotate on failure! */
-      if (MSG_IS_REGISTER(sip)) {
-        _eXosip_mark_registration_expired(excontext, sip->call_id->number);
-        if (eXosip_dnsutils_rotate_srv(&naptr_record->sipudp_record) > 0) {
-          OSIP_TRACE(osip_trace(__FILE__, __LINE__, OSIP_INFO1, NULL, "[eXosip] [UDP] [tid=%i] doing UDP failover [%s][%d] -> [%s][%d]\n", tid, host, port, naptr_record->sipudp_record.srventry[naptr_record->sipudp_record.index].srv,
-                                naptr_record->sipudp_record.srventry[naptr_record->sipudp_record.index].port));
-        }
-      }
+    if (MSG_IS_REGISTER(sip)) {
+      _eXosip_mark_registration_expired(excontext, sip->call_id->number);
     }
 
     return -1;
@@ -1464,18 +1290,11 @@ static int udp_tl_send_message(struct eXosip_t *excontext, osip_transaction_t *t
 
   i = (int) sendto(sock, (const void *) message, CAST_RECV_LEN(length), 0, (struct sockaddr *) &addr, len);
 
-  if (0 > i) {
+  if (i < 0) {
     char eb[ERRBSIZ];
     OSIP_TRACE(osip_trace(__FILE__, __LINE__, OSIP_INFO1, NULL, "[eXosip] [UDP] [tid=%i] [%s][%d] failure %s\n", tid, host, port, _ex_strerror(ex_errno, eb, ERRBSIZ)));
-    if (naptr_record != NULL) {
-      /* rotate on failure! */
-      if (MSG_IS_REGISTER(sip)) {
-        _eXosip_mark_registration_expired(excontext, sip->call_id->number);
-        if (eXosip_dnsutils_rotate_srv(&naptr_record->sipudp_record) > 0) {
-          OSIP_TRACE(osip_trace(__FILE__, __LINE__, OSIP_INFO1, NULL, "[eXosip] [UDP] [tid=%i] doing UDP failover [%s][%d] -> [%s][%d]\n", tid, host, port, naptr_record->sipudp_record.srventry[naptr_record->sipudp_record.index].srv,
-                                naptr_record->sipudp_record.srventry[naptr_record->sipudp_record.index].port));
-        }
-      }
+    if (MSG_IS_REGISTER(sip)) {
+      _eXosip_mark_registration_expired(excontext, sip->call_id->number);
     }
 
     /* SIP_NETWORK_ERROR; */
@@ -1504,12 +1323,8 @@ static int udp_tl_send_message(struct eXosip_t *excontext, osip_transaction_t *t
       now = osip_getsystemtime(NULL);
 
       if (tr != NULL && now - tr->birth_time > 10) {
-        /* avoid doing this twice... */
+        OSIP_TRACE(osip_trace(__FILE__, __LINE__, OSIP_INFO2, NULL, "[eXosip] [UDP] [tid=%i] [%s][%d] timeout\n", tid, host, port));
         _eXosip_mark_registration_expired(excontext, sip->call_id->number);
-        if (eXosip_dnsutils_rotate_srv(&naptr_record->sipudp_record) > 0) {
-          OSIP_TRACE(osip_trace(__FILE__, __LINE__, OSIP_INFO1, NULL, "[eXosip] [UDP] [tid=%i] doing UDP failover [%s][%d] -> [%s][%d]\n", tid, host, port, naptr_record->sipudp_record.srventry[naptr_record->sipudp_record.index].srv,
-                                naptr_record->sipudp_record.srventry[naptr_record->sipudp_record.index].port));
-        }
         osip_free(message);
         return -1;
       }
@@ -1556,6 +1371,9 @@ static int udp_tl_keepalive(struct eXosip_t *excontext) {
       if (sendto(reserved->udp_socket, (const void *) &jr->stun_binding, sizeof(jr->stun_binding), 0, (struct sockaddr *) &(jr->stun_addr), jr->stun_len) > 0) {
         OSIP_TRACE(osip_trace(__FILE__, __LINE__, OSIP_INFO1, NULL, "[eXosip] [UDP] [keepalive] STUN sent on UDP\n"));
         jr->ping_rfc5626 = osip_getsystemtime(NULL) + 9;
+      } else {
+        char eb[ERRBSIZ];
+        OSIP_TRACE(osip_trace(__FILE__, __LINE__, OSIP_INFO1, NULL, "[eXosip] [UDP] [keepalive] failure %s\n", _ex_strerror(ex_errno, eb, ERRBSIZ)));
       }
     }
   }
